import math
def racine_deux_balayage(nb_decimales):
    """détermine  par balayage un encadrement de racine carrée de 2 d’amplitude 10^(-n)"""
    pas = 10**(-nb_decimales)
    x = 0
    while x**2 < 2:
        x = x + pas
    return round(x - pas, nb_decimales), round(x, nb_decimales)

def racine_balayage(nombre,nb_decimales):
    """détermine  par balayage un encadrement de racine carrée de a d’amplitude 10^(-n)"""
    pas = 10**(-nb_decimales)
    x = 0
    compteur = 0
    while x**2 < nombre:
        x = x + pas
        compteur = compteur + 1
    return round(x-pas,nb_decimales),round(x,nb_decimales),compteur


def racine_balayage_opt(nombre, nb_decimales):
    """détermine  par balayage un encadrement de la racine carrée d'un nombre d’amplitude 10**(-nb_decimales), en optimisant la borne inférieure de l'encadrement à chaque tour de boucle"""
    debut = 0 # on crée une variable début qui recevra tour à tour les bornes inférieures de chaque encadrement
    compteur = 0
    for i in range(0,nb_decimales + 1) :
        pas = 10**(-i)
        x = debut # on commence le balayage à début et on reprend l'algorithme de balayage initial
        while x**2 < nombre :
            x = x  + pas
            compteur = compteur + 1
        debut = x - pas # pour le prochain balayage, on repartira avec la borne inférieure de l'encadrement obtenu
    return round(debut, nb_decimales), round(debut + pas, nb_decimales), compteur # on renvoie le dernier encadrement obtenu

def racine_dicho(nombre,nb_decimales):
    """détermine par dichotomie un encadrement de la racine carrée d'un nombre d'amplitude 10**(-nb_decimales) en partant d'un encadrement à l'unité a et b"""
    x = 0 # on reprend l'algorithme de balayage pour trouver un encadrement à l'unité de la racine carrée
    while x**2 < nombre :
        x = x + 1
    debut = x - 1
    fin = x
    compteur = 0
    while abs(fin-debut) > 10**(-nb_decimales):
        milieu = (debut + fin) / 2
        if milieu**2 <= nombre :
            debut = milieu
        else :
            fin = milieu
        compteur = compteur + 1
    return round(debut, nb_decimales), round(fin, nb_decimales), compteur

def racine_heron(a, nb_decimales) :
    """cette fonction calcule un encadrement de la racine carrée de a avec le nombre de décimales spécifié"""
    longueur = a
    largeur = 1
    compteur = 0
    while abs(longueur - largeur) > 10**(-nb_decimales) :
        longueur = (longueur + largeur) / 2
        largeur = a / longueur
        compteur = compteur + 1
    return round(largeur,nb_decimales), round(longueur,nb_decimales),compteur


def decoupage_tranches(nombre):
    """découpe un entier en blocs de 2 chiffres en partant de la droite"""
    chaine_nombre = str(nombre)
    liste_tranches = []
    while len(chaine_nombre) >= 2 :
        tranche = chaine_nombre[-2]+ chaine_nombre[-1] # extraction de la tranche la plus à droite
        liste_tranches = [tranche] + liste_tranches # ajout de la tranche en tête de liste
        chaine_nombre = chaine_nombre[:-2] # on prend la liste tronquée de ses deux derniers éléments
    if chaine_nombre != '': # on rajoute un éventuel reste si le nombre de chiffres est impair
        liste_tranches = [chaine_nombre] + liste_tranches
    return liste_tranches


def racine_potence(nombre,nb_decimales):
    """renvoie la racine carrée d'un nombre entier, avec une précision spécifiée par nb_decimales"""
    nombre = int(str(nombre)+'00'*nb_decimales) # reconstitution du nombre en l'y adjoignant des tranches de '00' pour aller jusqu'à la précision souhaitée
    liste_tranches = decoupage_tranches(nombre)
    # gestion de la première tranche
    dividende = int(liste_tranches.pop(0)) # on enlève la première tranche de la liste
    chiffre = 0
    while chiffre**2 <= dividende : # recherche de la racine carrée de la première tranche
        chiffre = chiffre + 1
    chiffre = chiffre - 1
    racine = chiffre
    reste = dividende - chiffre**2
    # gestion des autres tranches
    while liste_tranches != []:
        dividende = int(str(reste) + liste_tranches.pop(0))
        double = 2 * racine
        chiffre = 0
        while (double*10 + chiffre)*chiffre <= dividende :
            chiffre = chiffre + 1
        chiffre = chiffre - 1
        racine = 10*racine + chiffre
        reste = dividende - (double*10 + chiffre)*chiffre
    return racine/10**(nb_decimales)